library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity pwm_module is
	port(
		--informuje o ustawionej wartosci
		x : out std_logic_vector(3 downto 0) ;
		--wyjscie z sygnalem PWM
		pwm : out std_logic ;
		--sygnal zegarowy dla PWM
		CLK : in std_logic ;
		--wejscia zwieksz/zmniejsz wypelnienie
		INC, DEC : in std_logic
	);	    
end pwm_module;   

architecture pwm_moduleBehav of pwm_module is
	signal temp : unsigned (3 downto 0) ;
	signal counter_up : unsigned (3 downto 0) ;
	signal counter_down : unsigned (3 downto 0) ;
	signal INCprim, DECprim : std_logic ;
begin
	--filtracja drgan zestykow
	filtr: process (CLK) is
		variable presk : unsigned (16 downto 0)  ;
	begin
		if falling_edge(CLK) then
			presk := presk + 1 ;
			if presk = 0 then
				INCprim <= INC ;
				DECprim <= DEC  ;
			end if ;
		end if ;
	end process filtr ;

	--zliczanie sygnalow zegarowych
	pwmCounterEdge: process (CLK) is 
		variable pwmCounter : unsigned (3 downto 0) ;
	begin
		if falling_edge(CLK) then
			pwmCounter := pwmCounter + 1;
			if pwmCounter > 9 then
				pwmCounter := "0000" ;
			end if ;	 
		end if ;

		--komparator
		if  pwmCounter  <  temp  then
			pwm <= '1' ;
		else
			pwm <= '0' ;
		end if ;
	end process pwmCounterEdge ;

	--licznik programowalny - zwiekszenie zawartosic
	progCounterINC: process (INCprim) is begin
		--wykonanie polecenia
		if falling_edge(INCprim) and (temp < 9) then
			counter_up <= temp + 1;
		end if ;
	end process progCounterINC;

	--licznik programowalny - zwiekszenie zawartosic
	progCounterDEC: process (DECprim) is begin
		--wykonanie polecenia
		if falling_edge(DECprim) and (temp > 0) then
			counter_down <= temp - 1;
		end if ;
	end process progCounterDEC;

	--przypisania
	temp <= 
		counter_up   when (INCprim = '0') else
		counter_down when (DECprim = '0') else
		"0000" when temp>9 ;
	
	--przypisanie do wektora x
	x <= std_logic_vector(temp) ;
end pwm_moduleBehav;
